Package: future
Version: 1.26.1
Title: Unified Parallel and Distributed Processing in R for Everyone
Imports: digest, globals (>= 0.15.0), listenv (>= 0.8.0), parallel,
        parallelly (>= 1.30.0), tools, utils
Suggests: methods, RhpcBLASctl, R.rsp, markdown
VignetteBuilder: R.rsp
Authors@R: c(person("Henrik", "Bengtsson", role=c("aut", "cre", "cph"),
                                          email = "henrikb@braju.com"))
Description: The purpose of this package is to provide a lightweight and
    unified Future API for sequential and parallel processing of R
    expression via futures.  The simplest way to evaluate an expression
    in parallel is to use `x %<-% { expression }` with `plan(multisession)`.
    This package implements sequential, multicore, multisession, and
    cluster futures.  With these, R expressions can be evaluated on the
    local machine, in parallel a set of local machines, or distributed
    on a mix of local and remote machines.
    Extensions to this package implement additional backends for
    processing futures via compute cluster schedulers, etc.
    Because of its unified API, there is no need to modify any code in order
    switch from sequential on the local machine to, say, distributed
    processing on a remote compute cluster.
    Another strength of this package is that global variables and functions
    are automatically identified and exported as needed, making it
    straightforward to tweak existing code to make use of futures.
License: LGPL (>= 2.1)
LazyLoad: TRUE
ByteCompile: TRUE
URL: https://future.futureverse.org,
        https://github.com/HenrikBengtsson/future
BugReports: https://github.com/HenrikBengtsson/future/issues
RoxygenNote: 7.2.0
NeedsCompilation: no
Packaged: 2022-05-27 17:43:10 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph]
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2022-05-27 19:20:02 UTC
