# Generated by roxygen2: do not edit by hand

S3method("$<-",Future)
S3method("[",FutureCluster)
S3method("[",FutureGlobals)
S3method("[",sessionDetails)
S3method("[[",FutureCluster)
S3method(all.equal,FutureStrategyList)
S3method(all.equal,future)
S3method(as.FutureGlobals,FutureGlobals)
S3method(as.FutureGlobals,Globals)
S3method(as.FutureGlobals,list)
S3method(as.character,FutureResult)
S3method(as.raster,Mandelbrot)
S3method(c,FutureGlobals)
S3method(cancel,Future)
S3method(cancel,default)
S3method(cancel,environment)
S3method(cancel,list)
S3method(cancel,listenv)
S3method(conditionMessage,FutureCondition)
S3method(futures,environment)
S3method(futures,list)
S3method(futures,listenv)
S3method(getExpression,Future)
S3method(getFutureBackendConfigs,ClusterFuture)
S3method(getFutureBackendConfigs,Future)
S3method(getFutureBackendConfigs,MulticoreFuture)
S3method(getFutureBackendConfigs,UniprocessFuture)
S3method(interruptFuture,ClusterFutureBackend)
S3method(interruptFuture,FutureBackend)
S3method(interruptFuture,MulticoreFutureBackend)
S3method(journal,Future)
S3method(journal,FutureJournal)
S3method(journal,FutureJournalCondition)
S3method(journal,list)
S3method(launchFuture,ClusterFutureBackend)
S3method(launchFuture,FutureBackend)
S3method(launchFuture,MulticoreFutureBackend)
S3method(launchFuture,MultiprocessFutureBackend)
S3method(launchFuture,SequentialFutureBackend)
S3method(listFutures,FutureBackend)
S3method(listFutures,MultiprocessFutureBackend)
S3method(listFutures,SequentialFutureBackend)
S3method(mandelbrot,matrix)
S3method(mandelbrot,numeric)
S3method(nbrOfFreeWorkers,"NULL")
S3method(nbrOfFreeWorkers,ClusterFutureBackend)
S3method(nbrOfFreeWorkers,MulticoreFutureBackend)
S3method(nbrOfFreeWorkers,MultiprocessFutureBackend)
S3method(nbrOfFreeWorkers,SequentialFutureBackend)
S3method(nbrOfFreeWorkers,future)
S3method(nbrOfFreeWorkers,logical)
S3method(nbrOfFreeWorkers,multiprocess)
S3method(nbrOfWorkers,"NULL")
S3method(nbrOfWorkers,ClusterFutureBackend)
S3method(nbrOfWorkers,MulticoreFutureBackend)
S3method(nbrOfWorkers,MultiprocessFutureBackend)
S3method(nbrOfWorkers,SequentialFutureBackend)
S3method(nbrOfWorkers,future)
S3method(nbrOfWorkers,multiprocess)
S3method(plot,Mandelbrot)
S3method(print,ClusterFutureBackend)
S3method(print,Future)
S3method(print,FutureBackend)
S3method(print,FutureCondition)
S3method(print,FutureJournal)
S3method(print,FutureJournalSummary)
S3method(print,FutureResult)
S3method(print,FutureStrategy)
S3method(print,FutureStrategyList)
S3method(print,MultisessionFutureBackend)
S3method(print,future)
S3method(print,sessionDetails)
S3method(reset,Future)
S3method(reset,default)
S3method(reset,environment)
S3method(reset,list)
S3method(resetWorkers,default)
S3method(resetWorkers,multicore)
S3method(resolve,Future)
S3method(resolve,FutureGlobals)
S3method(resolve,default)
S3method(resolve,environment)
S3method(resolve,list)
S3method(resolve,listenv)
S3method(resolved,ClusterFuture)
S3method(resolved,Future)
S3method(resolved,FutureGlobals)
S3method(resolved,MulticoreFuture)
S3method(resolved,UniprocessFuture)
S3method(resolved,default)
S3method(resolved,environment)
S3method(resolved,list)
S3method(result,ClusterFuture)
S3method(result,ConstantFuture)
S3method(result,Future)
S3method(result,MulticoreFuture)
S3method(result,UniprocessFuture)
S3method(run,ConstantFuture)
S3method(run,Future)
S3method(run,UniprocessFuture)
S3method(stopWorkers,ClusterFutureBackend)
S3method(stopWorkers,FutureBackend)
S3method(stopWorkers,MulticoreFutureBackend)
S3method(stopWorkers,SequentialFutureBackend)
S3method(summary,FutureJournal)
S3method(tweak,"function")
S3method(tweak,character)
S3method(tweak,future)
S3method(unique,FutureGlobals)
S3method(validateFutureGlobals,FutureBackend)
S3method(value,Future)
S3method(value,environment)
S3method(value,list)
S3method(value,listenv)
S3method(with,FutureStrategyList)
export("%->%")
export("%<-%")
export("%conditions%")
export("%globals%")
export("%label%")
export("%lazy%")
export("%packages%")
export("%plan%")
export("%seed%")
export("%stdout%")
export("%tweak%")
export(ClusterFutureBackend)
export(ConnectionMisuseFutureCondition)
export(ConnectionMisuseFutureError)
export(ConnectionMisuseFutureWarning)
export(DefaultDeviceMisuseFutureCondition)
export(DefaultDeviceMisuseFutureError)
export(DefaultDeviceMisuseFutureWarning)
export(DeviceMisuseFutureCondition)
export(DeviceMisuseFutureError)
export(DeviceMisuseFutureWarning)
export(Future)
export(FutureBackend)
export(FutureCanceledError)
export(FutureCondition)
export(FutureDroppedError)
export(FutureError)
export(FutureGlobals)
export(FutureInterruptError)
export(FutureJournalCondition)
export(FutureLaunchError)
export(FutureMessage)
export(FutureResult)
export(FutureWarning)
export(GlobalEnvMisuseFutureCondition)
export(GlobalEnvMisuseFutureError)
export(GlobalEnvMisuseFutureWarning)
export(MulticoreFutureBackend)
export(MultiprocessFuture)
export(MultiprocessFutureBackend)
export(MultisessionFutureBackend)
export(RngFutureCondition)
export(RngFutureError)
export(RngFutureWarning)
export(SequentialFutureBackend)
export(UnexpectedFutureResultError)
export(UniprocessFuture)
export(as.FutureGlobals)
export(availableCores)
export(availableWorkers)
export(backtrace)
export(cancel)
export(cluster)
export(future)
export(futureAssign)
export(futureCall)
export(futureOf)
export(futureSessionInfo)
export(futures)
export(getExpression)
export(getGlobalsAndPackages)
export(interruptFuture)
export(launchFuture)
export(listFutures)
export(makeClusterPSOCK)
export(mandelbrot)
export(mandelbrot_tiles)
export(minifuture)
export(multicore)
export(multisession)
export(nbrOfFreeWorkers)
export(nbrOfWorkers)
export(plan)
export(reset)
export(resetWorkers)
export(resolve)
export(resolved)
export(result)
export(run)
export(sequential)
export(sessionDetails)
export(stopWorkers)
export(supportsMulticore)
export(tweak)
export(validateFutureGlobals)
export(value)
if (getRversion() >= "4.4") S3method(print,FutureCluster)
if (getRversion() >= "4.4") S3method(recvData,FutureNode)
if (getRversion() >= "4.4") S3method(sendData,FutureNode)
if (getRversion() >= "4.4") export(FUTURE)
if (getRversion() >= "4.4") export(makeClusterFuture)
if (getRversion() >= "4.4") importFrom(parallel,recvData)
if (getRversion() >= "4.4") importFrom(parallel,sendData)
importFrom(digest,digest)
importFrom(globals,Globals)
importFrom(globals,as.Globals)
importFrom(globals,cleanup)
importFrom(globals,findGlobals)
importFrom(globals,globalsByName)
importFrom(globals,globalsOf)
importFrom(globals,packagesOf)
importFrom(globals,walkAST)
importFrom(grDevices,as.raster)
importFrom(grDevices,hsv)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(listenv,as.listenv)
importFrom(listenv,get_variable)
importFrom(listenv,listenv)
importFrom(listenv,mapping)
importFrom(listenv,parse_env_subset)
importFrom(parallel,clusterCall)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,nextRNGStream)
importFrom(parallel,nextRNGSubStream)
importFrom(parallel,stopCluster)
importFrom(parallelly,as.cluster)
importFrom(parallelly,availableCores)
importFrom(parallelly,availableWorkers)
importFrom(parallelly,cloneNode)
importFrom(parallelly,connectionId)
importFrom(parallelly,isConnectionValid)
importFrom(parallelly,isNodeAlive)
importFrom(parallelly,killNode)
importFrom(parallelly,makeClusterPSOCK)
importFrom(parallelly,makeNodePSOCK)
importFrom(parallelly,serializedSize)
importFrom(parallelly,supportsMulticore)
importFrom(tools,md5sum)
importFrom(utils,capture.output)
importFrom(utils,file_test)
importFrom(utils,getS3method)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,sessionInfo)
importFrom(utils,str)
importFrom(utils,tail)
