#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-cluster,worker-termination.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
future:::testme("cluster,worker-termination")
