#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-nested_futures,mc.cores.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
future:::testme("nested_futures,mc.cores")
