% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/backend_api-03.MultiprocessFutureBackend-class.R
\name{MultiprocessFuture-class}
\alias{MultiprocessFuture-class}
\alias{MultiprocessFuture}
\title{A multiprocess future is a future whose value will be resolved asynchronously in a parallel process}
\usage{
MultiprocessFuture(expr = NULL, substitute = TRUE, envir = parent.frame(), ...)
}
\arguments{
\item{expr}{An \R \link[base]{expression}.}

\item{substitute}{If TRUE, argument \code{expr} is
\code{\link[base]{substitute}()}:d, otherwise not.}

\item{envir}{The \link{environment} from where global objects should be
identified.}

\item{\ldots}{Additional named elements passed to \code{\link[=Future]{Future()}}.}
}
\value{
\code{MultiprocessFuture()} returns an object of class \code{MultiprocessFuture}.
}
\description{
A multiprocess future is a future whose value will be resolved asynchronously in a parallel process
}
\keyword{internal}
